package org.jvending.registry.test;

import org.jvending.registry.RepositoryRegistry;
import org.jvending.registry.PropertyRepository;
import org.jvending.registry.Repository;
import org.jvending.testjar.ATestClass;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Set;

import junit.framework.TestCase;

public class RepositoryTest extends TestCase {

    private PropertyRepository fileRepository;

    private MyXmlRepository fileXmlRepository;

    private PropertyRepository fileRepository2;

    private MyXmlRepository fileXmlRepository2;

    private PropertyRepository jarRepository;

    private MyXmlRepository jarXmlRepository;

    private RepositoryRegistry repositoryRegistry = RepositoryRegistry.Factory.create();

    private static String baseDir = System.getProperty("basedir");

    public void setUp() {
        try {

            repositoryRegistry.loadFromFile(baseDir + "/target/test-classes/sample-config-1.xml");
            repositoryRegistry.loadFromFile(baseDir + "/target/test-classes/sample-config-2.xml");
            repositoryRegistry.loadFromResource("/config/sample-config-jar.xml", ATestClass.class);

            fileRepository = (PropertyRepository) repositoryRegistry.find("adapter");
            fileXmlRepository = (MyXmlRepository) repositoryRegistry.find("xml-sample");

            fileRepository2 = (PropertyRepository) repositoryRegistry.find("adapter-2");
            fileXmlRepository2 = (MyXmlRepository) repositoryRegistry.find("xml-sample-2");

            jarRepository = (PropertyRepository) repositoryRegistry.find("adapter-jar");
            jarXmlRepository = (MyXmlRepository) repositoryRegistry.find("xml-sample-jar");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            fail("Registry Config File not found");
        } catch(IOException e) {
            e.printStackTrace();
            fail("Registry Config File not found");
        }
    }

    public void testNonExistentRepository() {
        Repository repository = repositoryRegistry.find("bogus");
        assertNull("Returned value for bogus repository", repository);
    }

    public void testDoubleLoadingFromSameFile() {
        try {
            repositoryRegistry.loadFromFile(baseDir + "/target/test-classes/sample-config-1.xml");
        } catch(IOException e) {
            e.printStackTrace();
            fail("Double loading caused a failure");
        }
    }

    public void testLoadingAConfigThatIsNotXML() {
        try {
            repositoryRegistry.loadFromFile(baseDir + "/target/test-classes/notevenxml.xml");
        } catch(IOException e) {
            System.err.println("The following error is expected:" + e.getMessage());
            return;
        } catch(Exception e) {
            fail("Throwing unknown exception for bogus file: Message = " + e.getMessage());
        }
        fail("Somehow loaded a bogus registry file.");
    }

    public void testFlatProperty() {
        assertEquals("File Repository 1 failed", fileRepository.getValue("myprop"), "test");
        assertEquals("File Repository 2 failed", fileRepository2.getValue("myprop"), "test");
        assertEquals("Jar Repository failed", jarRepository.getValue("myprop"), "test-jar");
    }

    public void testXmlProperty() {
        assertEquals("Xml Repository 1 failed", fileXmlRepository.getMyProp(), "prop1");
        assertEquals("Xml Repository 2 failed", fileXmlRepository2.getMyProp(), "prop1");
        assertEquals("Jar Xml Repository failed", jarXmlRepository.getMyProp(), "prop1-jar");
    }

    public void testNonExistentRegistryConfigFile() {
        try {
            repositoryRegistry.loadFromFile(baseDir + "/target/test-classes/bogus.xml");
        } catch (FileNotFoundException e) {
            System.err.println("The following error is expected:" + e.getMessage());
            return;
        } catch(IOException e) {
            fail("Application is loading when no file is present.");
        }
        fail("Application is loading when no file is present.");
    }

    public void testNonExistentRepositoryFiles() {
        try {
            repositoryRegistry.loadFromFile(baseDir + "/target/test-classes/config-with-missing-files.xml");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            fail("Registry Config File not found");
        } catch(IOException e) {
            System.err.println("The following error is expected:" + e.getMessage());
            return;
        }
    }

    public void testNonExistentRepositoryClasses() {
        try {
            repositoryRegistry.loadFromFile(baseDir + "/target/test-classes/config-with-missing-classes.xml");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            fail("Registry Config File not found");
        } catch(IOException e) {
            System.err.println("The following error is expected:" + e.getMessage());
            return;
        }
    }

    public void testRepositoryNames() {
        Set names = repositoryRegistry.getRepositoryNames();
        assertEquals("Wrong number of repositories in the registry", 6, names.size());
    }

    public void testClear() {
        repositoryRegistry.empty();
        Set names = repositoryRegistry.getRepositoryNames();
        assertEquals("Repositories has elements in it even after emptying", 0, names.size());
    }

    public void testBadRepo() {
        try {
            repositoryRegistry.loadFromFile(baseDir + "/target/test-classes/config-with-bad-repo.xml");
        } catch(IOException e) {
            return;
        } catch(Exception e) {
            fail("Throwing unknown exception for bad repo file: Message = " + e.getMessage());
        }
        fail("Somehow processed a registry-config file that contains an invalid repo.");
    }



}
